from runregistry.attributes import (
    run_table_attributes,
    run_triplet_attributes,
    run_rr_attributes,
    run_oms_attributes,
    dataset_table_attributes,
    dataset_attributes,
    dataset_triplet_attributes,
)


def __parse_runs_arg(runs):
    """
    Helper function to parse runs arguments.
    Returns a list.
    """
    if isinstance(runs, int):
        return [runs]
    elif isinstance(runs, str):
        try:
            runs = int(runs)
            return [runs]
        except Exception:
            return []
    elif isinstance(runs, list):
        return runs
    else:
        return []


def transform_to_rr_run_filter(run_filter):
    """
    Transforms a filter to a compatible filter that RR back end understands.
    :param run_filter: a filter that the user inputs into the api client
    :return: returns a filter that runregistry back end understands.
    """
    if not run_filter or not isinstance(run_filter, dict):
        return {}
    transformed_filter = {}
    for key, value in run_filter.items():
        # If the value is not a dictionary (it is not in the form {'=': 323455}), so we assume the user just wanted simple '=', so we transform it from {'run_number': 323455} to {'run_number': {'=': 323455}}
        if not isinstance(value, dict):
            value = {"=": value}
        # If the user inputs a list of run_numbers in 'or' e.g.: {'run_number': {'or': [333333, 322222, 344444]}},
        # we want to translate it to {'run_number': {'or': [{'=':333333}, {'=':322222}, {'=':344444}]}}
        if isinstance(value, dict) and "or" in value and isinstance(value["or"], list):
            value["or"] = [
                {"=": x} if type(x) in (int, float, str) else x for x in value["or"]
            ]
        if key in run_table_attributes:
            transformed_filter[key] = value
        elif key in run_rr_attributes:
            transformed_filter["rr_attributes." + key] = value
        elif key in run_triplet_attributes:
            if "=" in value and isinstance(value["="], str):
                # if it is a string, we know for sure its either GOOD, BAD, STANDBY, ETC...
                value = value["="].upper()
                if value not in (
                    "GOOD",
                    "BAD",
                    "STANDBY",
                    "EXCLUDED",
                    "NOTSET",
                    "EMPTY",
                ):
                    raise Exception(
                        "status must be either GOOD, BAD, STANDBY, EXCLUDED, NOTSET or EMPTY"
                    )
                transformed_filter["triplet_summary." + key + "." + value] = {">": 0}
            # If it is not a string, it must be a filter for comments, or causes:
            # TODO: DO THE FILTER FOR COMMENTS, AND STATUSES
        elif key in run_oms_attributes:
            transformed_filter["oms_attributes." + key] = value
        # If user is performing an arbitrary filter, do not interfere:
        elif (
            key.startswith("rr_attributes")
            or key.startswith("oms_attributes")
            or key.startswith("triplet_summary")
        ):
            transformed_filter[key] = value
        else:
            raise Exception(
                "filter attribute {} not found within the listed attributes in runregistry/attributes.py".format(
                    key
                )
            )
    return transformed_filter


def transform_to_rr_dataset_filter(dataset_filter):
    """
    Transforms a filter to a compatible filter that RR back end understands.
    :param dataset_filter: a filter that the user inputs into the api client
    :return: returns a filter that runregistry back end understands.
    """
    if not dataset_filter or not isinstance(dataset_filter, dict):
        return {}
    transformed_filter = {}
    for key, value in dataset_filter.items():
        # If the value is not a dictionary (it is not in the form {'=': 323455}), so we assume the user just wanted simple '=', so we transform it from {'run_number': 323455} to {'run_number': {'=': 323455}}
        if not isinstance(value, dict) and key not in dataset_table_attributes:
            value = {"=": value}
        # If the user inputs a list of run_numbers in 'or' e.g.: {'run_number': {'or': [333333, 322222, 344444]}},
        # we want to translate it to {'run_number': {'or': [{'=':333333}, {'=':322222}, {'=':344444}]}}
        if isinstance(value, dict) and "or" in value and isinstance(value["or"], list):
            value["or"] = [
                {"=": x} if type(x) in (int, float, str) else x for x in value["or"]
            ]
        if key in dataset_table_attributes:
            if key == "dataset_name":
                # RR uses 'name' as identifier of dataset_name
                key = "name"
            transformed_filter[key] = value
        elif key in dataset_attributes:
            transformed_filter["dataset_attributes." + key] = value
        elif key in dataset_triplet_attributes:
            if isinstance(value["="], str):
                # if it is a string, we know for sure its either GOOD, BAD, STANDBY, ETC...
                value = value["="].upper()
            if value not in ("GOOD", "BAD", "STANDBY", "EXCLUDED", "NOTSET", "EMPTY"):
                raise Exception(
                    "status must be either GOOD, BAD, STANDBY, EXCLUDED, NOTSET or EMPTY"
                )
            transformed_filter["triplet_summary." + key + "." + value] = {">": 0}
        elif key in run_rr_attributes:
            transformed_filter["rr_attributes." + key] = value
        elif key in run_oms_attributes:
            transformed_filter["oms_attributes." + key] = value
        else:
            raise Exception(
                "filter attribute {} not found within the listed attributes in runregistry/attributes.py. If this attribute does exist, please make a PR to this client.".format(
                    key
                )
            )
    return transformed_filter



import pytest
import logging
import runregistry

logger = logging.getLogger(__name__)
VALID_RUN_NUMBER = 362874
VALID_DATASET_NAME = "/PromptReco/Commissioning2021/DQM"
EGROUPS_ERROR = "User needs to be part of any of the following e-groups"


@pytest.fixture
def setup_runregistry():
    logger.info("Connecting to development runregistry")
    runregistry.setup("development")


def test_advanced_move_datasets(setup_runregistry):
    with pytest.raises(Exception) as e:
        answers = runregistry.move_datasets(
            from_=runregistry.WAITING_DQM_GUI_CONSTANT,
            to_="OPEN",
            dataset_name=VALID_DATASET_NAME,
            run=VALID_RUN_NUMBER,
            workspace="global",
        )
    assert EGROUPS_ERROR in e.exconly()
    # TODO: Run also with a token that has permission
    with pytest.raises(Exception) as e:
        # Requires permission
        answers = runregistry.move_datasets(
            from_="OPEN",
            to_="SIGNOFF",
            dataset_name=VALID_DATASET_NAME,
            run=VALID_RUN_NUMBER,
            workspace="ctpps",
        )
    assert EGROUPS_ERROR in e.exconly()


def test_advanced_move_datasets_bad_from(setup_runregistry):
    with pytest.raises(ValueError):
        answers = runregistry.move_datasets(
            from_="MPAMIES",
            to_="SIGNOFF",
            dataset_name=VALID_DATASET_NAME,
            run=VALID_RUN_NUMBER,
            workspace="global",
        )


def test_advanced_move_datasets_no_run(setup_runregistry):
    with pytest.raises(ValueError):
        answers = runregistry.move_datasets(
            from_="MPAMIES",
            to_="SIGNOFF",
            dataset_name=VALID_DATASET_NAME,
            workspace="global",
        )


def test_advanced_move_datasets_bad_to(setup_runregistry):
    with pytest.raises(ValueError):
        answers = runregistry.move_datasets(
            from_=runregistry.WAITING_DQM_GUI_CONSTANT,
            to_="AGKINARES",
            dataset_name=VALID_DATASET_NAME,
            run=VALID_RUN_NUMBER,
            workspace="global",
        )


def test_advanced_make_significant_single_run(setup_runregistry):
    # Get latest run in dev runregistry and make it significant
    run = runregistry.get_runs(limit=1, filter={})[0]
    with pytest.raises(Exception) as e:
        # requires permission
        runregistry.make_significant_runs(run=run["run_number"])
    assert EGROUPS_ERROR in e.exconly()


def test_advanced_make_significant_multi_runs(setup_runregistry):
    # Get latest run in dev runregistry and make it significant
    run = runregistry.get_runs(limit=1, filter={})[0]
    with pytest.raises(Exception) as e:
        # requires permission
        runregistry.make_significant_runs(runs=[run["run_number"]])
    assert EGROUPS_ERROR in e.exconly()


def test_advanced_make_significant_no_runs(setup_runregistry):
    run = runregistry.get_runs(limit=1, filter={})[0]
    with pytest.raises(ValueError):
        # Required args missing
        runregistry.make_significant_runs()


def test_advanced_reset_RR_attributes_and_refresh_runs_signed_off(setup_runregistry):
    with pytest.raises(Exception) as e:
        # Cannot refresh runs which are not open
        answers = runregistry.reset_RR_attributes_and_refresh_runs(
            runs=VALID_RUN_NUMBER
        )
    assert "Run must be in state OPEN" in repr(e)


def test_advanced_reset_RR_attributes_and_refresh_runs_no_run(setup_runregistry):
    with pytest.raises(ValueError) as e:
        # Cannot refresh runs which are not open
        answers = runregistry.reset_RR_attributes_and_refresh_runs()


def test_advanced_reset_RR_attributes_and_refresh_runs_open(setup_runregistry):
    run = runregistry.get_runs(limit=1, filter={})[0]
    answers = runregistry.reset_RR_attributes_and_refresh_runs(runs=run["run_number"])
    assert isinstance(answers, list)


def test_advanced_manually_refresh_components_statuses_for_no_runs(setup_runregistry):
    with pytest.raises(ValueError):
        # Missing argument
        runregistry.manually_refresh_components_statuses_for_runs()


def test_advanced_manually_refresh_components_statuses_for_runs_open(setup_runregistry):
    run = runregistry.get_runs(limit=1, filter={})[0]
    # Currently, manual refresh does not need special permissions??
    answers = runregistry.manually_refresh_components_statuses_for_runs(
        runs=run["run_number"]
    )
    assert isinstance(answers, list)


def test_advanced_manually_refresh_components_statuses_for_runs_signed_off(
    setup_runregistry,
):
    with pytest.raises(Exception) as e:
        runregistry.manually_refresh_components_statuses_for_runs(runs=VALID_RUN_NUMBER)
    assert "Run must be in state OPEN" in e.exconly()


def test_advanced_move_runs_no_run_arg(setup_runregistry):
    with pytest.raises(ValueError):
        # Raises ValueError
        runregistry.move_runs("OPEN", "SIGNOFF")


def test_advanced_move_single_run(setup_runregistry):
    with pytest.raises(Exception) as e:
        # Requires permission
        runregistry.move_runs("OPEN", "SIGNOFF", run=VALID_RUN_NUMBER)
    assert "User needs to be part of any of the following e-groups" in e.exconly()


def test_advanced_move_multi_runs(setup_runregistry):
    with pytest.raises(Exception) as e:
        # Requires permission
        runregistry.move_runs("OPEN", "SIGNOFF", runs=[VALID_RUN_NUMBER])
    assert EGROUPS_ERROR in e.exconly()


def test_advanced_move_runs_invalid(setup_runregistry):
    with pytest.raises(ValueError):
        # Requires permission
        runregistry.move_runs("!!!!", "???", runs=[VALID_RUN_NUMBER])


def test_advanced_edit_rr_lumisections_good(setup_runregistry):
    with pytest.raises(Exception) as e:
        # Requires permission
        runregistry.edit_rr_lumisections(
            VALID_RUN_NUMBER, 0, 1, "castor-castor", "GOOD"
        )
    assert "User needs to be part of any of the following e-groups" in e.exconly()


def test_advanced_edit_rr_lumisections_bad(setup_runregistry):
    with pytest.raises(ValueError):
        # Requires permission
        runregistry.edit_rr_lumisections(
            VALID_RUN_NUMBER, 0, 1, "castor-castor", "KALHSPERA STHN PAREA ;)"
        )


def test_advanced_change_run_class_list(setup_runregistry):
    with pytest.raises(Exception) as e:
        # Requires permission
        runregistry.change_run_class(run_numbers=[VALID_RUN_NUMBER], new_class="test")
    assert "User needs to be part of any of the following e-groups" in e.exconly()


def test_advanced_change_run_class_int(setup_runregistry):
    """
    Current behavior is to accept both a list and an int as run_numbers
    """
    with pytest.raises(Exception) as e:
        # Still Requires permission
        runregistry.change_run_class(run_numbers=VALID_RUN_NUMBER, new_class="test")
    assert "User needs to be part of any of the following e-groups" in e.exconly()


def test_advanced_change_run_class_list_with_bad_run_types(setup_runregistry):
    with pytest.raises(ValueError) as e:
        runregistry.change_run_class(run_numbers=["3455555"], new_class="test")


def test_advanced_change_run_class_bad_run_numbers(setup_runregistry):
    with pytest.raises(ValueError):
        runregistry.change_run_class(run_numbers="3455555", new_class="test")


def test_advanced_change_run_class_bad_new_class_type(setup_runregistry):
    with pytest.raises(ValueError):
        # Requires permission
        runregistry.change_run_class(run_numbers=[VALID_RUN_NUMBER], new_class=1)



import sys
import pytest
import requests
from runregistry.utils import (
    transform_to_rr_run_filter,
    transform_to_rr_dataset_filter,
    dataset_triplet_attributes,
    run_rr_attributes,
)
from runregistry.attributes import (
    run_oms_attributes,
    run_triplet_attributes,
    dataset_attributes,
)
from runregistry.runregistry import (
    __version__,
    _get_user_agent,
    _get_headers,
    _get_token,
    _get_target,
    __parse_runs_arg as _parse_runs_arg,
    setup,
)


class TestRunRegistryFilterCreation:
    def test_get_run(self):
        run_number = 323434
        assert transform_to_rr_run_filter(run_filter={"run_number": run_number}) == {
            "run_number": {"=": run_number}
        }

    def test_transform_triplets(self):
        VALID_VALUES = ["GOOD", "BAD", "STANDBY", "EXCLUDED", "NOTSET", "EMPTY"]
        for attribute in run_triplet_attributes:
            for value in VALID_VALUES:
                result = transform_to_rr_run_filter(
                    run_filter={attribute: {"=": value}}
                )
                assert f"triplet_summary.{attribute}.{value}" in result
        with pytest.raises(Exception):
            transform_to_rr_run_filter(run_filter={"ecal-ecal": {"=": "HEHEHE"}})

    def test_transform_invalid_attribute(self):
        with pytest.raises(Exception):
            transform_to_rr_run_filter(run_filter={"BRE KAKOS MPELAS": {"=": "HEHEHE"}})
        assert not transform_to_rr_run_filter(None)
        assert not transform_to_rr_run_filter("")
        assert not transform_to_rr_run_filter("SDF")
        assert not transform_to_rr_run_filter(15)

    def test_transform_attributes(self):
        VALID_ATTRIBUTES = [
            "rr_attributes",
            "oms_attributes",
            "triplet_summary",
            "triplet_summaryalksjdflkajsd",  # Unsure why this should be supported
        ]
        FILTER = {"=": "aaa"}
        for attribute in VALID_ATTRIBUTES:
            result = transform_to_rr_run_filter(run_filter={attribute: FILTER})
            assert attribute in result and result[attribute] == FILTER

    def test_transform_run_oms_attributes(self):

        FILTER = {"=": "test"}
        for attribute in run_oms_attributes:
            if attribute == "run_number":
                print(
                    "run_number seems to exist on both run_oms_attributes and "
                    + "run_triplet_attributes, meaning that run_number cannot "
                    + "be used with run_oms_attributes, only triplets"
                )
                continue
            result = transform_to_rr_run_filter(run_filter={attribute: FILTER})
            assert (
                f"oms_attributes.{attribute}" in result
                and result[f"oms_attributes.{attribute}"] == FILTER
            )

    def test_transform_run_rr_attributes(self):
        FILTER = {"=": "test"}
        for attribute in run_rr_attributes:
            result = transform_to_rr_run_filter(run_filter={attribute: FILTER})
            assert (
                f"rr_attributes.{attribute}" in result
                and result[f"rr_attributes.{attribute}"] == FILTER
            )

    def test_get_multiple_run_using_or(self):
        run_number1 = 323555
        run_number2 = 323444
        run_number3 = 343222
        run_number4 = 333333
        user_input = {
            "run_number": {
                "or": [run_number1, run_number2, run_number3, {"=": run_number4}]
            }
        }
        desired_output = {
            "run_number": {
                "or": [
                    {"=": run_number1},
                    {"=": run_number2},
                    {"=": run_number3},
                    {"=": run_number4},
                ]
            }
        }

        assert transform_to_rr_run_filter(run_filter=user_input) == desired_output


class TestDatasetFilterCreation:
    def test_dataset_attributes(self):
        FILTER = {"=": "aaa"}
        for attribute in dataset_attributes:
            result = transform_to_rr_dataset_filter(dataset_filter={attribute: FILTER})
            assert (
                f"dataset_attributes.{attribute}" in result
                and result[f"dataset_attributes.{attribute}"] == FILTER
            )

    def test_dataset_triplet_attributes_valid(self):
        VALID_VALUES = ["GOOD", "BAD", "STANDBY", "EXCLUDED", "NOTSET", "EMPTY"]
        for value in VALID_VALUES:
            for attribute in dataset_triplet_attributes:
                result = transform_to_rr_dataset_filter(
                    dataset_filter={attribute: {"=": value}}
                )
                assert f"triplet_summary.{attribute}.{value}" in result

    def test_dataset_triplet_attributes_invalid(self):
        with pytest.raises(Exception):
            transform_to_rr_dataset_filter(
                dataset_filter={dataset_triplet_attributes[0]: {"=": "ZE MALESI :("}}
            )

    def test_transform_run_oms_attributes(self):
        FILTER = {"=": "test"}
        for attribute in run_oms_attributes:
            if attribute == "run_number":
                print(
                    "run_number seems to exist on both run_oms_attributes and "
                    + "run_triplet_attributes, meaning that run_number cannot "
                    + "be used with run_oms_attributes, only triplets"
                )
                continue
            result = transform_to_rr_dataset_filter(dataset_filter={attribute: FILTER})
            assert (
                f"oms_attributes.{attribute}" in result
                and result[f"oms_attributes.{attribute}"] == FILTER
            )

    def test_transform_run_rr_attributes(self):
        FILTER = {"=": "test"}
        for attribute in run_rr_attributes:
            result = transform_to_rr_dataset_filter(dataset_filter={attribute: FILTER})
            assert (
                f"rr_attributes.{attribute}" in result
                and result[f"rr_attributes.{attribute}"] == FILTER
            )

    def test_transform_invalid_attribute(self):
        with pytest.raises(Exception):
            transform_to_rr_dataset_filter(
                dataset_filter={"BRE KAKOS MPELAS": {"=": "HEHEHE"}}
            )
        assert not transform_to_rr_dataset_filter(None)
        assert not transform_to_rr_dataset_filter("")
        assert not transform_to_rr_dataset_filter("SDF")
        assert not transform_to_rr_dataset_filter(15)


class TestUtils:
    MAGIC_RUN = 255525

    def test_user_agent(self):
        ua = _get_user_agent()
        assert (
            __version__ in ua
            and "runregistry_api_client" in ua
            and f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"
            in ua
            and requests.__version__ in ua
            and "zodiac sign" not in ua
        )

    def test_runregistry_setup(self):
        for target in ["development", "local", "production"]:
            setup(target)

            assert _get_target() == target

        with pytest.raises(Exception):
            setup("HAHAHAHA >:)")

    def test_headers(self):
        headers = _get_headers(token="WHATEVER :/")
        assert all(
            [key in headers for key in ["User-Agent", "Authorization", "Content-type"]]
        )

    def test_get_token(self):
        setup("local")
        assert _get_token() == ""

    def test_parse_runs_int(self):
        runs = _parse_runs_arg(self.MAGIC_RUN)
        assert isinstance(runs, list) and runs[0] == self.MAGIC_RUN

    def test_parse_runs_str_int(self):
        runs = _parse_runs_arg(str(self.MAGIC_RUN))
        assert isinstance(runs, list) and runs[0] == self.MAGIC_RUN

    def test_parse_runs_str_str(self):
        runs = _parse_runs_arg("LMAO ://////////")
        assert isinstance(runs, list) and len(runs) == 0

    def test_parse_runs_list_int(self):
        runs = _parse_runs_arg([self.MAGIC_RUN, self.MAGIC_RUN + 1])
        assert isinstance(runs, list) and len(runs) == 2

    def test_parse_runs_list_str(self):
        # This case should probably be fixed to only accept list of ints
        runs = _parse_runs_arg([str(self.MAGIC_RUN), str(self.MAGIC_RUN + 1)])
        assert isinstance(runs, list) and len(runs) == 2

    def test_parse_runs_dict(self):
        runs = _parse_runs_arg({self.MAGIC_RUN})
        assert isinstance(runs, list) and len(runs) == 0



run_class_name = "Collisions18"
dataset_name = "/Express/Collisions2018/DQM"
ls_query = "(l.CTPPS45_READY = '1' or l.CTPPS56_READY = '1')"
output_filename = "test.json"


query = "((r.RDA_CMP_RP45_210 = 'GOOD' and r.RDA_CMP_RP45_220 = 'GOOD') or \
        (r.RDA_CMP_RP45_CYL = 'GOOD' and r.RDA_CMP_RP45_210 = 'GOOD') or \
        (r.RDA_CMP_RP45_CYL = 'GOOD' and r.RDA_CMP_RP45_220 = 'GOOD')) or \
        ((r.RDA_CMP_RP56_210 = 'GOOD' and r.RDA_CMP_RP56_220 = 'GOOD') or \
        (r.RDA_CMP_RP56_220 = 'GOOD' and r.RDA_CMP_RP56_CYL = 'GOOD') or \
        (r.RDA_CMP_RP56_210 = 'GOOD' and r.RDA_CMP_RP56_CYL = 'GOOD'))"


import sys
from rhapi import DEFAULT_URL, RhApi


def get_data(q):
    api = RhApi(DEFAULT_URL, debug=False)

    p = {"class": "Collisions18"}
    qid = api.qid(q)

    return api.csv(q, p)


def obtain_json_from_rr(run_class_name, dataset_name, query, ls_query, output_filename):
    run_numbers_query = "select r.RUN_NUMBER from runreg_ctpps.datasets r where \
   r.run_class_name = '{0}' and r.RDA_NAME = '{1}' {2}".format(
        run_class_name, dataset_name, "and (" + query + ")" if query else ""
    )

    run_numbers = [
        int(x)
        for x in [
            y for y in get_data(run_numbers_query).split("\n") if y and y.isdigit()
        ]
    ]

    with open(output_filename, "w") as f:
        f.write("{")
        sign = ","
        for i, run_number in enumerate(run_numbers):
            lumisections_numbers = "select l.RDR_SECTION_FROM, l.RDR_SECTION_TO from runreg_ctpps.dataset_lumis l \
           where l.RDR_RUN_NUMBER = {0} {1}".format(
                run_number, "and (" + ls_query + ")" if ls_query else ""
            )
            if i == len(run_numbers) - 1:
                sign = "} "
            f.write(
                '"{0}": [[{1}]]{2}\n'.format(
                    run_number,
                    "], [".join(
                        [x for x in get_data(lumisections_numbers).split("\n") if x][1:]
                    ),
                    sign,
                )
            )



import json
import pytest
import runregistry
from runregistry.runregistry import (
    get_run,
    get_runs,
    get_dataset_names_of_run,
    get_dataset,
    get_datasets,
    get_lumisections,
    get_oms_lumisections,
    get_lumisection_ranges,
    get_oms_lumisection_ranges,
    get_joint_lumisection_ranges,
    get_cycles,
    get_lumisection_ranges_by_component,
    get_datasets_accepted,
    create_json,
    setup,
)

VALID_RUN_NUMBER = 327743
VALID_RUN_RANGE_START = 309000
VALID_RUN_RANGE_STOP = 310000

INVALID_RUN_NUMBER = 420420420
VALID_DATASET_NAME = "/PromptReco/HICosmics18A/DQM"
INVALID_DATASET_NAME = "/PromptKikiriko/HELLOCosmics18Z/DQM"


def test_client_get_run():
    run_number = VALID_RUN_NUMBER
    run = get_run(run_number=VALID_RUN_NUMBER)
    assert run["run_number"] == VALID_RUN_NUMBER
    # Non-existent run
    run_number = INVALID_RUN_NUMBER
    run = get_run(run_number=run_number)
    assert not run


def test_client_get_runs_no_filter():
    with pytest.raises(Exception) as e:
        runs = get_runs()
    assert "must pass a filter" in e.exconly()


def test_client_get_runs():
    # Gets runs between run number VALID_RUN_RANGE_START and VALID_RUN_RANGE_STOP
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        }
    }
    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    # Gets runs that contain lumisections that classified DT as GOOD AND lumsiections that classified hcal as STANDBY
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "dt-dt": "GOOD",
        # 'hcal': 'STANDBY'
    }

    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    runs = []

    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "tracker-strip": "GOOD",
    }
    runs = get_runs(filter=filter_run)
    print(json.dumps(runs))
    assert len(runs) > 0


def test_client_get_runs_with_ignore_filter():
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "oms_attributes.hlt_key": {"like": "%commissioning2018%"},
        "triplet_summary.dt-dt.GOOD": {">": 0},
    }
    runs = get_runs(filter=filter_run, ignore_filter_transformation=True)
    assert len(runs) > 0


def test_client_get_datasets_with_ignore_filter():
    # datasets = get_datasets(filter={
    #     "run_number": {
    #         "and": [{
    #             ">": VALID_RUN_RANGE_START
    #         }, {
    #             "<": VALID_RUN_RANGE_STOP
    #         }]
    #     },
    #     "oms_attributes.hlt_key": {
    #         "like": "%commissioning2018%"
    #     },
    #     "triplet_summary.dt-dt.GOOD": {
    #         ">": 0
    #     },
    # },
    #                         ignore_filter_transformation=True)

    datasets = get_datasets(
        filter={
            "and": [
                {
                    "run_number": {
                        "and": [
                            {">": VALID_RUN_RANGE_START},
                            {"<": VALID_RUN_RANGE_STOP},
                        ]
                    }
                }
            ],
            "name": {"and": [{"<>": "online"}]},
            "dataset_attributes.global_state": {
                "and": [{"or": [{"=": "OPEN"}, {"=": "SIGNOFF"}, {"=": "COMPLETED"}]}]
            },
        },
        ignore_filter_transformation=True,
    )
    assert len(datasets) > 0


# test_client_get_datasets_with_ignore_filter()

# test_client_get_runs_with_ignore_filter()


def test_client_get_runs_not_compressed():
    runs = get_runs(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            },
            "dt-dt": "GOOD",
        },
        compress_attributes=False,
    )
    assert len(runs) > 0


def get_runs_with_combined_filter():
    runs = get_runs(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
                # },
                # 'hlt_key': {
                #     'like': '%commissioning2018%'
                # },
                # 'significant': {
                #     '=': True
            }
        }
    )
    assert len(runs) > 0


def test_client_get_dataset_names_of_run():
    dataset_names = get_dataset_names_of_run(run_number=VALID_RUN_NUMBER)
    assert len(dataset_names) > 0


def test_client_get_dataset():
    dataset = get_dataset(run_number=VALID_RUN_NUMBER, dataset_name=VALID_DATASET_NAME)
    assert dataset["run_number"] == VALID_RUN_NUMBER
    assert dataset["name"] == VALID_DATASET_NAME
    dataset = get_dataset(
        run_number=INVALID_RUN_NUMBER, dataset_name=INVALID_DATASET_NAME
    )
    assert not dataset


def test_client_get_datasets_no_limit():
    datasets = get_datasets(
        compress_attributes=True,
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            }
        },
    )
    assert len(datasets) > 0
    assert "Run" not in datasets[0]


def test_client_get_datasets_no_compression():
    datasets = get_datasets(
        compress_attributes=False,
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            }
        },
    )
    assert len(datasets) > 0
    assert "Run" in datasets[0]


def test_client_get_datasets_with_limit():
    datasets = get_datasets(
        limit=5,
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            }
        },
    )
    assert len(datasets) > 0


def test_client_get_datasets_no_filters():
    with pytest.raises(Exception) as e:
        datasets = get_datasets()
    assert "must pass a filter" in e.exconly()


def test_client_get_lumisections():
    lumisections = get_lumisections(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_client_get_oms_lumisections():
    lumisections = get_oms_lumisections(VALID_RUN_NUMBER)
    assert len(lumisections) > 0
    dataset_lumisections = get_oms_lumisections(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(dataset_lumisections) > 0


def test_client_get_lumisection_ranges():
    lumisections = get_lumisection_ranges(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_client_get_oms_lumisection_ranges():
    lumisections = get_oms_lumisection_ranges(VALID_RUN_NUMBER)
    assert len(lumisections) > 0


def test_client_get_joint_lumisection_ranges():
    lumisections = get_joint_lumisection_ranges(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_client_get_collisions18():
    runs = get_runs(filter={"class": "Collisions18"})
    assert len(runs) > 0


def test_client_get_or_run():
    runs = get_runs(filter={"run_number": {"or": [VALID_RUN_NUMBER]}})


def test_client_get_datasets_with_filter():
    datasets = get_datasets(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            },
            "tracker-strip": "GOOD",
        }
    )
    assert len(datasets) > 0


json_logic = {
    "and": [
        {">=": [{"var": "run.oms.energy"}, 6000]},
        {"<=": [{"var": "run.oms.energy"}, 7000]},
        {">=": [{"var": "run.oms.b_field"}, 3.7]},
        {"in": ["25ns", {"var": "run.oms.injection_scheme"}]},
        {"==": [{"in": ["WMass", {"var": "run.oms.hlt_key"}]}, False]},
        {"==": [{"var": "lumisection.rr.dt-dt"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.csc-csc"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.l1t-l1tmu"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.l1t-l1tcalo"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.hlt-hlt"}, "GOOD"]},
        {"==": [{"var": "lumisection.oms.bpix_ready"}, True]},
    ]
}


def test_client_create_json():
    json = create_json(
        json_logic=json_logic, dataset_name_filter="/PromptReco/Collisions2018A/DQM"
    )
    print(json)


def test_client_custom_filter():
    filter_arg = {
        "dataset_name": {"like": "%/PromptReco/Cosmics18CRUZET%"},
        "run_number": {
            "and": [{">=": VALID_RUN_RANGE_START}, {"<=": VALID_RUN_RANGE_STOP}]
        },
        "class": {"like": "Cosmics18CRUZET"},
        "global_state": {"like": "COMPLETED"},
        "ecal-ecal": "EXCLUDED",
    }

    datasets = get_datasets(filter=filter_arg)
    assert datasets


def test_client_setup_random():
    with pytest.raises(Exception) as e:
        setup("Olo kaskarikes mas kaneis")
    assert "Invalid setup target" in e.exconly()


def test_client_setup_production():
    setup("production")
    assert "https://cmsrunregistry" in runregistry.runregistry.api_url


def test_client_setup_development():
    setup("development")
    assert "https://dev-cmsrunregistry" in runregistry.runregistry.api_url


def test_client_get_cycles():
    answers = get_cycles()
    assert len(answers) > 0
    assert "id_cycle" in answers[0]
    assert "cycle_name" in answers[0]
    assert "cycle_attributes" in answers[0]
    assert "deadline" in answers[0]
    assert "datasets" in answers[0]


def test_client_get_lumisection_ranges_by_component():
    answer = get_lumisection_ranges_by_component(VALID_RUN_NUMBER)
    assert len(answer.keys()) > 0
    assert "dt-dt" in answer
    assert "rpc-hv" in answer
    assert "tracker-track_private" in answer


def test_client_get_datasets_accepted():
    answers = get_datasets_accepted()
    assert len(answers) > 0
    assert isinstance(answers[0], dict)
    assert "regexp" in answers[0]
    assert "id" in answers[0]
    assert "name" in answers[0]
    assert "class" in answers[0]



import codecs
import os
import re
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), "r") as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


with open(os.path.join(here, "README.md")) as f:
    long_description = f.read()

setup(
    name="runregistry",
    version=find_version("runregistry", "runregistry.py"),
    packages=find_packages(),
    author="Fabio Espinosa",
    description="CMS Run Registry client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cms-DQM/runregistry_api_client/",
    author_email="f.e@cern.ch",
    maintainer="CMS DQM team",
    maintainer_email="cms-dqm-coreTeam@cern.ch",
    install_requires=["requests", "cernrequests", "python-dotenv"],
)



import os
import sys
import time
import json
import requests
from dotenv import load_dotenv
from cernrequests import get_api_token
from runregistry.utils import (
    transform_to_rr_run_filter,
    transform_to_rr_dataset_filter,
    __parse_runs_arg,
)

__version__ = "3.1.1"

# Look for .env file in the directory of the caller
# first. If it exists, use it.
if os.path.exists(os.path.join(os.getcwd(), ".env")):
    load_dotenv(dotenv_path=os.path.join(os.getcwd(), ".env"))
else:
    load_dotenv()


# Silence unverified HTTPS warning:
# urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
PAGE_SIZE = 50

# Offline table
WAITING_DQM_GUI_CONSTANT = "waiting dqm gui"

# Valid Lumisection statuses
LUMISECTION_STATES = ["GOOD", "BAD", "STANDBY", "EXCLUDED", "NOTSET"]

ONLINE_RUN_STATES = ["SIGNOFF", "OPEN", "COMPLETED"]

OFFLINE_DATASET_STATES = ["SIGNOFF", "OPEN", "COMPLETED", WAITING_DQM_GUI_CONSTANT]

# Time to sleep between JSON creation checks
JSON_CREATION_SLEEP_TIME = 15

staging_cert = ""
staging_key = ""
api_url = ""
use_cookies = True
email = "api@api"
client_id = os.environ.get("SSO_CLIENT_ID")
client_secret = os.environ.get("SSO_CLIENT_SECRET")
target_application = ""
target_name = ""


def setup(target):
    global api_url
    global target_application
    global use_cookies
    global target_name

    if target == "local":
        api_url = "http://localhost:9500"
        use_cookies = False
        target_application = ""
    elif target == "development":
        api_url = "https://dev-cmsrunregistry.web.cern.ch/api"
        use_cookies = True
        target_application = "dev-cmsrunregistry-sso-proxy"
    elif target == "production":
        api_url = "https://cmsrunregistry.web.cern.ch/api"
        use_cookies = True
        target_application = "cmsrunregistry-sso-proxy"
    else:
        raise Exception(
            f'Invalid setup target "{target}". Valid options: "local", "development", "production".'
        )
    target_name = target


def _get_user_agent():
    return f"runregistry_api_client/{__version__} ({_get_target()}, python {sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}, requests {requests.__version__})"


def _get_headers(token: str = ""):
    headers = {"Content-type": "application/json"}
    if not use_cookies:
        headers["email"] = email
    if token:
        headers["Authorization"] = "Bearer " + token
    headers["User-Agent"] = _get_user_agent()
    return headers


setup(os.environ.get("ENVIRONMENT", "production"))


def _get_target():
    return target_name


def _get_token():
    """
    Gets the token required to query RR API through the CERN SSO.
    :return: the token required to query Run Registry API.
    """
    if _get_target() == "local":
        return ""
    token, expiration_date = get_api_token(
        client_id=client_id,
        client_secret=client_secret,
        target_application=target_application,
    )
    return token


def _get_page(
    url, page=0, data_type="runs", ignore_filter_transformation=False, **kwargs
):
    """
    :param ignore_filter_transformation: If user knows how the filter works (by observing http requests on RR website), and wants to ignore the suggested transformation to query API, user can do it by setting ignore_filter_transformation to True
    :param filter: The filter to be transformed into RR syntax, and then sent for querying
    :return: A page in Run registry
    """
    headers = _get_headers(token=_get_token())
    query_filter = kwargs.pop("filter", {})
    if data_type == "runs" and not ignore_filter_transformation:
        query_filter = transform_to_rr_run_filter(run_filter=query_filter)
    elif data_type == "datasets" and not ignore_filter_transformation:
        query_filter = transform_to_rr_dataset_filter(dataset_filter=query_filter)
    if _get_target() in ["development", "local"]:
        print(url)
        print(query_filter)
    payload = json.dumps(
        {
            "page": page,
            "filter": query_filter,
            "page_size": kwargs.pop("page_size", PAGE_SIZE),
            "sortings": kwargs.pop("sortings", []),
        }
    )

    return requests.post(url, headers=headers, data=payload).json()


def get_dataset_names_of_run(run_number, **kwargs):
    """
    Gets the existing dataset names of a run_number
    :return: Array of dataset names of the specified run_number
    """
    url = "{}/get_all_dataset_names_of_run/{}".format(api_url, run_number)
    return requests.get(url, headers=_get_headers(token=_get_token())).json()


def get_run(run_number, **kwargs):
    """
    Gets all the info about a particular run
    :param run_number: run_number of specified run
    """
    run = get_runs(filter={"run_number": run_number}, **kwargs)
    if not run:
        return {}
    if len(run) > 1:
        raise Exception(
            f"Unexpected number of results returned for run {run_number} ({len(run)}), was expecting exactly 1"
        )
    return run[0]


def get_runs(limit=40000, compress_attributes=True, **kwargs):
    """
    Gets all runs that match the filter given in
    :param compress_attributes: Gets the attributes inside rr_attributes:* and the ones in the DatasetTripletCache (The lumisections insdie the run/dataset) and spreads them over the run object
    :param filter: the filter applied to the runs needed
    """
    url = "{}/runs_filtered_ordered".format(api_url)
    initial_response = _get_page(url=url, data_type="runs", page=0, **kwargs)
    if "err" in initial_response:
        raise ValueError(initial_response["err"])

    resource_count = initial_response["count"]
    page_count = initial_response["pages"]
    runs = initial_response["runs"]
    if resource_count > limit:
        print(
            "ALERT: The specific run registry api request returns more runs than the limit({}), consider passing a greater limit to get_runs(limit=number) to get the whole result.".format(
                limit
            )
        )
    if resource_count > 10000:
        print(
            "WARNING: fetching more than 10,000 runs from run registry. you probably want to pass a filter into get_runs, or else this will take a while."
        )
    if resource_count > 20000 and "filter" not in kwargs:
        raise Exception(
            "ERROR: For run registry queries that retrieve more than 20,000 runs, you must pass a filter into get_runs, an empty filter get_runs(filter={}) works"
        )
    for page_number in range(1, page_count):
        additional_runs = _get_page(
            page=page_number, url=url, data_type="runs", **kwargs
        )
        runs.extend(additional_runs.get("runs"))
        if len(runs) >= limit:
            runs = runs[:limit]
            break

    if compress_attributes:
        compressed_runs = []
        for run in runs:
            compressed_run = {
                "oms_attributes": run["oms_attributes"],
                **run["rr_attributes"],
                "lumisections": run["DatasetTripletCache"]["triplet_summary"],
                **run,
            }
            del compressed_run["rr_attributes"]
            del compressed_run["DatasetTripletCache"]
            compressed_runs.append(compressed_run)
        return compressed_runs

    return runs


def get_dataset(run_number, dataset_name="online", **kwargs):
    """
    Gets information about the dataset specified by run_number and dataset_name
    :param run_number:  The run number of the dataset
    :param dataset_name: The name of the dataset. 'online' is the dataset of the online run. These are Run Registry specific dataset names e.g. online, /PromptReco/Collisions2018D/DQM, /Express/Collisions2018/DQM
    """
    dataset = get_datasets(
        filter={"run_number": run_number, "dataset_name": dataset_name}, **kwargs
    )
    if not dataset:
        return {}
    if len(dataset) > 1:
        raise Exception(
            f"Unexpected number of results returned for dataset {dataset_name} of run {run_number} ({len(dataset)}), was expecting exactly 1"
        )
    return dataset[0]


def get_datasets(limit=40000, compress_attributes=True, **kwargs) -> list:
    """
    Gets all datasets that match the filter given
    :param compress_attributes: Gets the attributes inside rr_attributes:* and the ones in the DatasetTripletCache (The lumisections insdie the run/dataset) and spreads them over the run object
    """
    url = "{}/datasets_filtered_ordered".format(api_url)
    initial_response = _get_page(url=url, data_type="datasets", page=0, **kwargs)
    if "err" in initial_response:
        raise ValueError(initial_response["err"])

    resource_count = initial_response["count"]
    page_count = initial_response["pages"]
    datasets = initial_response["datasets"]
    if resource_count > limit:
        print(
            "ALERT: The specific api request returns more datasets than the limit({}), consider passing a greater limit to get_datasets(limit=number) to get the whole result.".format(
                limit
            )
        )
    if resource_count > 10000:
        print(
            "WARNING: fetching more than 10,000 datasets. you probably want to pass a filter into get_datasets, or else this will take a while."
        )
    if resource_count > 20000 and "filter" not in kwargs:
        raise Exception(
            "ERROR: For queries that retrieve more than 20,000 datasets, you must pass a filter into get_datasets, an empty filter get_datasets(filter={}) works"
        )
    for page_number in range(1, page_count):
        additional_datasets = _get_page(
            page=page_number, url=url, data_type="datasets", **kwargs
        )
        datasets.extend(additional_datasets.get("datasets"))
        if len(datasets) >= limit:
            datasets = datasets[:limit]
            break

    if compress_attributes:
        compressed_datasets = []
        for dataset in datasets:
            compressed_dataset = {
                **dataset["Run"]["rr_attributes"],
                **dataset,
                "lumisections": dataset["DatasetTripletCache"]["triplet_summary"],
            }
            del compressed_dataset["DatasetTripletCache"]
            del compressed_dataset["Run"]
            compressed_datasets.append(compressed_dataset)
        return compressed_datasets
    return datasets


def get_cycles():
    url = "{}/cycles/global".format(api_url)
    headers = _get_headers(token=_get_token())
    if _get_target() in ["development", "local"]:
        print(url)
    return requests.get(url, headers=headers).json()


def _get_lumisection_helper(url, run_number, dataset_name="online", **kwargs):
    """
    Puts the headers and POST data for all other lumisection methods
    """

    headers = _get_headers(token=_get_token())
    payload = json.dumps({"run_number": run_number, "dataset_name": dataset_name})
    return requests.post(url, headers=headers, data=payload).json()


def get_lumisections(run_number, dataset_name="online", **kwargs):
    """
    Gets the Run Registry lumisections of the specified dataset
    """
    url = "{}/lumisections/rr_lumisections".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_oms_lumisections(run_number, dataset_name="online", **kwargs):
    """
    Gets the OMS lumisections saved in RR database
    """
    url = "{}/lumisections/oms_lumisections".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_lumisection_ranges(run_number, dataset_name="online", **kwargs):
    """
    Gets the lumisection ranges of the specified dataset. Returns
    a list of dicts, each one containing a lumisection "range", dictated
    by the 'start' and 'stop' keys of the dict. In the same dict,
    the status, cause, and comments per component are found.
    """
    url = "{}/lumisections/rr_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_lumisection_ranges_by_component(run_number, dataset_name="online", **kwargs):
    """
    Gets the lumisection ranges of the specified dataset as a dict,
    where the components are the keys (e.g. 'rpc-rpc'). Each dict value is
    a list of lumisection "ranges" (dicts) for the specific component. The exact
    range is dictated by the 'start' and 'stop' keys of the nested dict.

    Similar to get_lumisection_ranges, but organized by component.
    """
    url = "{}/lumisections/rr_lumisection_ranges_by_component".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


def get_oms_lumisection_ranges(run_number, **kwargs):
    """
    Gets the OMS lumisection ranges of the specified dataset (saved in RR database)
    """
    url = "{}/lumisections/oms_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name="online", **kwargs)


def get_joint_lumisection_ranges(run_number, dataset_name="online", **kwargs):
    """
    Gets the lumisection ranges of the specified dataset, breaken into RR breaks and OMS ranges
    """
    url = "{}/lumisections/joint_lumisection_ranges".format(api_url)
    return _get_lumisection_helper(url, run_number, dataset_name, **kwargs)


# Using json portal (safe):
def create_json(json_logic, dataset_name_filter, **kwargs):
    """
    It adds a json to the queue and polls until json is either finished or an error occured
    """
    if not isinstance(json_logic, str):
        json_logic = json.dumps(json_logic)
    url = "{}/json_portal/generate".format(api_url)

    headers = _get_headers(token=_get_token())
    payload = json.dumps(
        {"json_logic": json_logic, "dataset_name_filter": dataset_name_filter}
    )
    response = requests.post(url, headers=headers, data=payload).json()

    # Id of json:
    id_json = response["id"]
    # Poll JSON until job is complete
    while True:
        # polling URL:
        url = "{}/json_portal/json".format(api_url)

        headers = _get_headers(token=_get_token())

        payload = json.dumps({"id_json": id_json})
        response = requests.post(url, headers=headers, data=payload)
        if response.status_code == 200:
            return response.json()["final_json"]
        elif response.status_code == 202:
            # stil processing
            print(f"progress creating json: {response.json()['progress']}")
            time.sleep(JSON_CREATION_SLEEP_TIME)
        elif response.status_code == 203:
            # stil processing
            print("json process is submited and pending, please wait...")
            time.sleep(JSON_CREATION_SLEEP_TIME)
        else:
            raise Exception(
                f"Error {response.status_code} during JSON creation: {response.text}"
            )


def get_datasets_accepted():
    """
    Method for fetching current datasets accepted in Offline Run Registry
    """
    url = "{}/datasets_accepted".format(api_url)
    headers = _get_headers(token=_get_token())
    return requests.get(url, headers=headers).json()


# advanced RR operations ==============================================================================
# Online Table
def move_runs(from_, to_, run=None, runs=[], **kwargs):
    """
    move run/runs from one state to another
    """
    if not run and not runs:
        raise ValueError("move_runs(): no 'run' and 'runs' arguments were provided")

    if from_ not in ONLINE_RUN_STATES or to_ not in ONLINE_RUN_STATES:
        raise ValueError(
            f"move_runs(): got states '{from_}, '{to_}'",
            f" but allowed states are {ONLINE_RUN_STATES}",
        )

    url = "%s/runs/move_run/%s/%s" % (api_url, from_, to_)

    headers = _get_headers(token=_get_token())

    if run:
        runs = [run]

    answers = []
    for run_number in runs:
        payload = json.dumps({"run_number": run_number})
        answer = requests.post(url, headers=headers, data=payload)
        if answer.status_code != 200:
            raise Exception(
                f"Got response {answer.status_code} when moving datasets: {answer.text}"
            )
        answers.append(answer.json())

    return answers


def make_significant_runs(run=None, runs=[], **kwargs):
    """
    mark run/runs significant
    """
    if not run and not runs:
        raise ValueError(
            "make_significant_runs(): no 'run' and 'runs' arguments were provided"
        )

    url = "%s/runs/mark_significant" % (api_url)
    headers = _get_headers(token=_get_token())

    if run:
        runs = [run]

    answers = []
    for run_number in runs:
        data = {"run_number": run}
        answer = requests.post(url, headers=headers, json=data)
        if answer.status_code != 200:
            raise Exception(
                f"Got response {answer.status_code} when making runs significant: {answer.text}"
            )
        answers.append(answer.json())

    return answers


def reset_RR_attributes_and_refresh_runs(runs=[], **kwargs):
    """
    reset RR attributes and refresh run/runs
    """
    runs = __parse_runs_arg(runs)
    if not runs:
        raise ValueError(
            "reset_RR_attributes_and_refresh_runs(): no 'runs' argument was provided"
        )
    headers = _get_headers(token=_get_token())
    answers = []
    for run_number in runs:
        url = "%s/runs/reset_and_refresh_run/%d" % (api_url, run_number)
        answer = requests.post(url, headers=headers)
        if answer.status_code != 200:
            raise Exception(
                f"Got response {answer.status_code} when resetting and refreshing runs: {answer.text}"
            )
        answers.append(answer.json())

    return answers


def manually_refresh_components_statuses_for_runs(runs=[], **kwargs):
    """
    Refreshes all components statuses for the runs specified that have not been
    changed by shifters.
    """
    runs = __parse_runs_arg(runs)

    if not runs:
        raise ValueError(
            "manually_refresh_components_statuses_for_runs(): no 'runs' argument was provided"
        )

    headers = _get_headers(token=_get_token())
    answers = []
    for run_number in runs:
        url = "%s/runs/refresh_run/%d" % (api_url, run_number)
        answer = requests.post(url, headers=headers)
        if answer.status_code != 200:
            raise Exception(
                f"Got response {answer.status_code} when manually refreshing component statuses: {answer.text}"
            )
        answers.append(answer.json())

    return answers


def edit_rr_lumisections(
    run,
    lumi_start,
    lumi_end,
    component,
    status,
    comment="",
    cause="",
    dataset_name="online",
    **kwargs,
):
    """
    WIP edit RR lumisections attributes
    """
    if status not in LUMISECTION_STATES:
        raise ValueError(
            f"edit_rr_lumisections(): got status '{status}'",
            f" but allowed statuses are {LUMISECTION_STATES}",
        )

    url = "%s/lumisections/edit_rr_lumisections" % (api_url)

    headers = _get_headers(token=_get_token())
    payload = json.dumps(
        {
            "new_lumisection_range": {
                "start": lumi_start,
                "end": lumi_end,
                "status": status,
                "comment": comment,
                "cause": cause,
            },
            "run_number": run,
            "dataset_name": dataset_name,
            "component": component,
        }
    )
    answer = requests.put(url, headers=headers, data=payload)
    if answer.status_code != 200:
        raise Exception(
            f"Got response {answer.status_code} when editing rr lumisections: {answer.text}"
        )
    return answer.json()


def move_datasets(
    from_, to_, dataset_name, workspace="global", run=None, runs=[], **kwargs
):
    """
    Move offline dataset/datasets from one state to another.
    Requires a privileged token.
    """
    if not run and not runs:
        raise ValueError("move_datasets(): no 'run' and 'runs' arguments were provided")

    if from_ not in OFFLINE_DATASET_STATES or to_ not in OFFLINE_DATASET_STATES:
        raise ValueError(
            f"move_datasets(): got states '{from_}', '{to_}",
            f" but allowed states are {OFFLINE_DATASET_STATES}",
        )

    url = "%s/datasets/%s/move_dataset/%s/%s" % (api_url, workspace, from_, to_)

    headers = _get_headers(token=_get_token())

    if run:
        runs = [run]

    answers = []
    for run_number in runs:
        payload = json.dumps(
            {
                "run_number": run_number,
                "dataset_name": dataset_name,
                "workspace": workspace,
            }
        )
        answer = requests.post(url, headers=headers, data=payload)
        if answer.status_code != 200:
            raise Exception(
                f"Got response {answer.status_code} when moving datasets: {answer.text}"
            )
        answers.append(answer.json())

    return answers


def change_run_class(run_numbers, new_class):
    """
    Method for changing the class of a run (or runs),
    e.g. from "Commissioning22" to "Cosmics22".
    Requires a privileged token.
    """
    headers = _get_headers(token=_get_token())

    def _execute_request_for_single_run(run_number, new_class):
        payload = json.dumps({"class": new_class})
        return requests.put(
            url="%s/manual_run_edit/%s/class" % (api_url, run_number),
            headers=headers,
            data=payload,
        )

    if not isinstance(new_class, str):
        raise ValueError(f'Invalid input "{new_class}" for "new_class"')
    answers = []
    # If just one int provided, make it into a list
    if isinstance(run_numbers, int):
        run_numbers = [run_numbers]

    if isinstance(run_numbers, list):
        for run_number in run_numbers:
            if not isinstance(run_number, int):
                raise ValueError(
                    "Invalid run number value found in run_numbers. Please provide a list of numbers."
                )
            answer = _execute_request_for_single_run(run_number, new_class)
            if answer.status_code != 200:
                raise Exception(
                    f"Got response {answer.status_code} when changing run class: {answer.text}"
                )
            answers.append(answer.json())
    else:
        raise ValueError(
            'Invalid input for "run_numbers". Please provide a list of numbers.'
        )
    return answers
